/*
Scratch Board Emulator (Approximately)
 Adam Cooper, Feb 2012
 You may use this under the terms of the MIT Open Source licence.
 
 Follows the Scratch Board Tech Info R2  - http://info.scratch.mit.edu/sites/infoscratch.media.mit.edu/files/file/ScratchBoard_Tech_InfoR2.pdf
 Except that the channel assignments in that doc do not match Scratch!
 */

//modify the following lines according to which Arduino pins are used
//Analogue inputs
const int sliderPin = A0;//NB doubling up with resA
const int lightPin = A4;
const int soundPin = A5;
const int resAPin = A0;
const int resBPin = A1;
const int resCPin = A2;
const int resDPin = A3;
//digital input
const int buttonPin = 4;

//sound sensor settings
//duration of startup calibration in milliseconds (sets the zero of the transmitted value)
const int calibrateTime = 2000; 
//the max analogRead for the hardware. Used to rescale the transmitted value. Readings > this give 1023
const int maxReading = 600;
//how large should the moving average window be?
//the returned value is the arithmetic mean of the previous soundWindowLen readings
const int soundWindowLen = 4;//10 makes for rather slow response

//global variables
int audioThreshold;
float scaleFactor;
int soundWindow[soundWindowLen];
int soundWindowIndex;
int soundTotal;

void setup(){
  pinMode(buttonPin, INPUT);

  //calibrate sound sensor - be quiet when starting or "reset" to recalibrate!
  audioThreshold = 0;
  unsigned long endTime=calibrateTime+millis();
  while(millis()<endTime){
    if(analogRead(soundPin)>audioThreshold){
      audioThreshold = analogRead(soundPin);
    }
  }
  scaleFactor = 1023.0/float(maxReading - audioThreshold);

  //initialise the window for sound averaging
  soundWindowIndex=0;
  soundTotal=0;
  for(int i = 0;i++;i<soundWindowLen){
    soundWindow[i]=0;
  }

  /*open comms with Scratch
   NB: make sure the Arduino IDE is not competing with Scratch for use of the
   same COM port. Release the port in the IDE by changing to an incorrect port
   or quit the IDE then select the correct port in Scratch (right-click on the "monitor")
   */
  Serial.begin(38400);
}

void loop(){
  //
  if(Serial.available()>0){
    if (Serial.read() == 1){ 
      scratchResponse();
    }
  }
}

/*
Read sound value from analog input and scale linearly so that
 0 is returned when value below the max reading during the calibration period
 1023 is returned when the value is >= the maxReading constant
 */
int readScaledSound(){
  int rawRead=analogRead(soundPin);
  if(rawRead<=audioThreshold){
    return 0;
  }
  else{
    //rescale so a raw reading of maxReading => a sent value of 1023
    int scaledRead = int(scaleFactor * float(rawRead-audioThreshold));
    scaledRead=(scaledRead>1023)?1023:scaledRead;//in case maxReading was too low, set a ceiling
    return scaledRead;
  }
}

/*
takes a new sound reading as argument and returns the running average
 soundWindow is treated as a circular buffer
 */
int updateSoundWindow(int newVal){
  //update the total
  soundTotal-=soundWindow[soundWindowIndex];
  soundTotal+=newVal;
  //store the new value
  soundWindow[soundWindowIndex] = newVal;
  //increment the pointer, bringing back to 0 as required
  soundWindowIndex++;
  soundWindowIndex = soundWindowIndex%soundWindowLen;

  return soundTotal/soundWindowLen;
}

/*
Respond to a Scratch polling event with all sensor data
 */
void scratchResponse(){
  //according to the tech spec we need to send a firmware vsn first. A bogus firmware = 128+1
  transmitValue(15, 129);
  //now sensor values
  transmitValue(0, analogRead(resAPin));//resistance-A
  transmitValue(1, analogRead(resBPin));//resistance-B    
  transmitValue(2, analogRead(resCPin));//resistance-C
  transmitValue(3, analogRead(resDPin));//resistance-D
  transmitValue(4, analogRead(sliderPin));//slider
  transmitValue(5, analogRead(lightPin));//light
  transmitValue(6, updateSoundWindow(readScaledSound()));//sound
  transmitValue(7, digitalRead(buttonPin)?1023:0);//button
}

/*
Transmit the 2-bytes of data that identify the sensor and its value
 */
void transmitValue(int sensor, int value)
{
  //delays as per the scratchboard tech spec R2
  Serial.write( B10000000 | ((sensor & B1111)<<3) | ((value>>7) & B111));
  delayMicroseconds(400);
  Serial.write( value & B1111111);
  delayMicroseconds(400);
}


